# frozen_string_literal: true

require 'twitter_snowflake/version'
require 'twitter_snowflake/snowflake'

# TwitterSnowflake is a library to parse snowflake IDs.
module TwitterSnowflake
  # Twitter's epoch in milliseconds. Used as default epoch to parse IDs.
  TWITTER_EPOCH = 1_288_834_974_657

  # Timestamp offset (low-order bit of the timestamp collection)
  TIMESTAMP_BINARY_OFFSET = 22

  # Worker ID offset (low-order bit of the worker ID collection)
  WORKER_ID_BINARY_OFFSET = 17

  # Mask to extract worker ID.
  WORKER_ID_MASK = 0x3E000

  # Process ID offset (low-order bit of the process ID collection)
  PROCESS_ID_BINARY_OFFSET = 12

  # Mask to extract process ID.
  PROCESS_ID_MASK = 0x1F000

  # Mask to extract increment.
  INCREMENT_MASK = 0xFFF

  class << self
    # Creates a snowflake object for any given timestamp, worker ID, process ID and increment.
    #
    # @param epoch [Integer] base epoch in milliseconds to perform calculations.
    # @param timestamp [Integer] the timestamp in milliseconds.
    #
    # @return [Snowflake] the generated snowflake.
    def synthesize(timestamp:, worker_id: 0, process_id: 0, increment: 0, epoch: TWITTER_EPOCH)
      id  = (timestamp - epoch) << TIMESTAMP_BINARY_OFFSET
      id += (worker_id << WORKER_ID_BINARY_OFFSET)
      id += (process_id << PROCESS_ID_BINARY_OFFSET)
      id += increment

      Snowflake.new(id: id, epoch: epoch)
    end

    # Parses a snowflake ID.
    #
    # @param id [Integer] the snowflake ID.
    # @param epoch [Integer] base epoch in milliseconds to perform calculations.
    #
    # @return [Snowflake] a snowflake object.
    def parse(id, epoch: TWITTER_EPOCH)
      Snowflake.new(id: id, epoch: epoch)
    end

    # Extracts the timestamp from a snowflake ID.
    #
    # @param id [Integer] the snowflake ID.
    # @param epoch [Integer] base epoch in milliseconds to perform calculations.
    #
    # @return [Integer] the timestamp in milliseconds.
    def timestamp(id, epoch: TWITTER_EPOCH)
      (id >> TIMESTAMP_BINARY_OFFSET) + epoch
    end

    # Extracts the worker ID from a snowflake ID.
    #
    # @param id [Integer] the snowflake ID.
    #
    # @return [Integer] the worker ID.
    def worker_id(id)
      (id & WORKER_ID_MASK) >> WORKER_ID_BINARY_OFFSET
    end

    # Extracts the process ID from a snowflake ID.
    #
    # @param id [Integer] the snowflake ID.
    #
    # @return [Integer] the process ID.
    def process_id(id)
      (id & PROCESS_ID_MASK) >> PROCESS_ID_BINARY_OFFSET
    end

    # Extracts the increment from a snowflake ID.
    #
    # @param id [Integer] the snowflake ID.
    #
    # @return [Integer] the increment.
    def increment(id)
      id & INCREMENT_MASK
    end
  end
end
