# frozen_string_literal: true

module TwitterSnowflake
  # A snowflake ID as a Ruby object.
  class Snowflake
    # @return [Integer] the snowflake ID in its original form.
    attr_reader :id

    # @return [Integer] the base epoch in milliseconds to perform calculations.
    attr_reader :epoch

    # @return [Integer] the timestamp extracted from the ID in milliseconds.
    attr_reader :timestamp

    # @return [Integer] the worker ID extracted form the ID.
    attr_reader :worker_id

    # @return [Integer] the proccess ID extracted from the ID.
    attr_reader :process_id

    # @return [Integer] the increment extracted from the ID.
    attr_reader :increment

    # @return [Time] the timestamp as a Time object
    attr_reader :time

    # @param id [Integer] the ID itself.
    # @param epoch [Intger] base epoch in milliseconds to perform calculations.
    def initialize(id:, epoch:)
      # Data used during extraction process
      @id    = id
      @epoch = epoch

      # Extracting data
      @timestamp  = TwitterSnowflake.timestamp(@id, epoch: @epoch)
      @worker_id  = TwitterSnowflake.worker_id(@id)
      @process_id = TwitterSnowflake.process_id(@id)
      @increment  = TwitterSnowflake.increment(@id)

      # Useful information
      @time = Time.at(@timestamp / 1000.0)
    end

    # Checks if two snowflakes are equal. Two snowflakes are considered equal
    # if both have the same ID and are based on the same epoch.
    #
    # @param other [Snowflake] the other snowflake.
    #
    # @return [Boolean] whether the snowflakes are equal or not.
    def ==(other)
      (@id == other.id) && (@epoch == other.epoch)
    end

    # Checks if this snowflake has been generated before another snowflake.
    #
    # @param other [Snowflake] the other snowflake.
    #
    # @return [Boolean] whether this snowflake has been generated before another snowflake or not.
    def before?(other)
      @timestamp < other.timestamp
    end

    # Checks if this snowflake has been generated after another snowflake.
    #
    # @param other [Snowflake] the other snowflake.
    #
    # @return [Boolean] whether this snowflake has been generated after another snowflake or not.
    def after?(other)
      @timestamp > other.timestamp
    end
  end
end
